//
// Created by sofie on 24.11.22.
//
#include <cmath>
#include <ros/ros.h>

#ifndef ROCK_ANALYSIS_CALIBRATION_H
#define ROCK_ANALYSIS_CALIBRATION


/*
 * Cross calibrate the laser scanners: maximum intensity target and scale according to it,
 * so that the same target is maximum for all. Scales from 0 to 1;
 */
class Calibration{
public:

    double a;
    double b;
    bool set = false; 

    Calibration(){}

    void setValues(double minI, double maxI){

      a = 1/(maxI-minI);  //breaking down normalization formula of (value-min)/(max-min) to a*value+b
      b = 1 - a * maxI;
      set = true;

      
    }

    double calibrate(double value){ 
      if(set){
      return a * value + b; //from observed range [minI, maxI] to [0,1]
      }
      ROS_ERROR("No values for calibration set");
      ros::shutdown;
      return 0.0;
    }

};


#endif //ROCK_ANALYSIS_CALIBRATION_H
