//modified from https://www.geeksforgeeks.org/regression-analysis-and-the-best-fitting-line-using-c/

#include <vector>

using namespace std;

#ifndef ROCK_ANALYSIS_REGRESSION_H
#define ROCK_ANALYSIS_REGRESSION


class regression {

private:
    vector<double> x;
    vector<double> y;
    
    double sum_xy;
    double sum_x;
    double sum_y;
    double sum_x_square;
    double sum_y_square;

    bool set = false;

public:

    // best fitting line
    double coeff;
    double constTerm;

    regression(){}

    // Constructor to provide the default
    // values to all the terms in the
    // object of class regression
    regression(vector<double> x, vector<double> y)
    {
        this->x = x;
        this->y = y;

        for (int i = 0; i < y.size(); i++) {
            sum_xy += x[i] * y[i];
            sum_x += x[i];
            sum_y += y[i];
            sum_x_square += x[i] * x[i];
            sum_y_square += y[i] * y[i];
        }

        calculateCoefficient();
        calculateConstantTerm();

        set = true;
        
    }

    void setParams(vector<double> x, vector<double> y){
        this->x = x;
        this->y = y;

        for (int i = 0; i < y.size(); i++) {
            sum_xy += x[i] * y[i];
            sum_x += x[i];
            sum_y += y[i];
            sum_x_square += x[i] * x[i];
            sum_y_square += y[i] * y[i];
        }

        calculateCoefficient();
        calculateConstantTerm();

        set = true;
    }

    // Function to predict the value
    // corresponding to some input
    float predict(float x)
    {   if(set){ return coeff * x + constTerm;}
        else {
            std::cout << "Error: regression nor calculated yet, set parameters" << endl;
            return 0;
        }
    }

    // Function that returns overall
    // sum of square of errors
    float errorSquare()
    {
        if(set){
            float ans = 0;
            for (int i = 0;
                i < x.size(); i++) {
                ans += ((predict(x[i]) - y[i])
                        * (predict(x[i]) - y[i]));
            }
            return ans;
        } else {
            std::cout << "Error: regression nor calculated yet, set parameters" << endl;
            return 0;
        }
    }

    private:
 
    // Function that calculate the coefficient/
    // slope of the best fitting line
    void calculateCoefficient()
    {
        int N = x.size();
        double numerator = (N * sum_xy - sum_x * sum_y);
        double denominator = (N * sum_x_square - sum_x * sum_x);
        coeff = numerator / denominator;
    }
 
    // Member function that will calculate
    // the constant term of the best
    // fitting line
    void calculateConstantTerm()
    {
        int N = x.size();
        double numerator = (sum_y * sum_x_square - sum_x * sum_xy);
        double denominator = (N * sum_x_square - sum_x * sum_x);
        constTerm = numerator / denominator;
    }

};

#endif //ROCK_ANALYSIS_REGRESSION_H